@@ -7,8 +7,6 @@
 class MissingContent(Exception):
     """If a user creates a code-include directive with no target."""
 
-    pass
-
 
 class MissingNamespace(Exception):
     """If the user specifies a missing target.
@@ -19,8 +17,6 @@ class MissingNamespace(Exception):
 
     """
 
-    pass
-
 
 class MissingTag(Exception):
     """If the user specifies a missing directive.
@@ -31,16 +27,14 @@ class MissingTag(Exception):
 
     """
 
-    pass
-
 
 class NotFoundFile(Exception):
     """An class to describe a local intersphinx project that does not exist."""
 
-    pass
-
 
 class NotFoundUrl(Exception):
     """An class to describe a URL intersphinx project that does not exist."""
 
-    pass
+
+class NoMatchFound(Exception):
+    """The directive and namespace were correct but no source code could be found."""
