@@ -6,9 +6,12 @@
 
 import os
 import textwrap
+import typing
 import unittest
+from unittest import mock
 
-from six.moves import mock
+import bs4
+from docutils import nodes as nodes_
 
 from code_include import error_classes
 from code_include import extension
@@ -26,29 +29,31 @@ class Inputs(unittest.TestCase):
     @mock.patch("code_include.source_code._get_app_inventory")
     def _test(
         self,
-        content,
-        exception_class,
-        _get_app_inventory,
-        _get_source_code_from_object,
-        _reraise_exception,
-    ):
+        content: list[str],
+        exception_class: typing.Type[Exception],
+        _get_app_inventory: mock.MagicMock,
+        _get_source_code_from_object: mock.MagicMock,
+        _reraise_exception: mock.MagicMock,
+    ) -> None:
         """A generic function that checks a code-include directive for issues.
 
         Args:
-            content (list[str]):
+            content:
                 The lines that the user provides in a standard code-include block.
-            exception_class (Exception):
+            exception_class:
                 The exception that should be raised, when given `content`.
-            _get_app_inventory (:class:`mock.mock.MagicMock`):
+            _get_app_inventory:
                 The function that's normally used to query a Sphinx
                 project's inventory to find every HTML file-path and
                 tag-able header.
-            _reraise_exception (:class:`mock.mock.MagicMock`):
+            _reraise_exception:
                 A function that must be set to return `True` so that
                 this test will forcibly raise the found exception.
 
         """
-        data = common.load_cache(os.path.join(_CURRENT_DIRECTORY, "fake_project", "objects.inv"))
+        data = common.load_cache(
+            os.path.join(_CURRENT_DIRECTORY, "fake_project", "objects.inv")
+        )
 
         _get_app_inventory.return_value = data
         _reraise_exception.return_value = True
@@ -59,23 +64,23 @@ class Inputs(unittest.TestCase):
         with self.assertRaises(exception_class):
             directive.run()
 
-    def test_no_required_argument(self):
+    def test_no_required_argument(self) -> None:
         """Check that missing content raises the expected exception."""
         self._test(  # pylint: disable=no-value-for-parameter
             [""], error_classes.MissingContent
         )
 
-    def test_incorrect_directive_target(self):
+    def test_incorrect_directive_target(self) -> None:
         """Check that a bad tag like ":foo:" raises the expected exception."""
         self._test(  # pylint: disable=no-value-for-parameter
-            [u":nonexistent:tag:`some.module.that.may.exist`"],
+            [":nonexistent:tag:`some.module.that.may.exist`"],
             error_classes.MissingTag,
         )
 
-    def test_incorrect_namespace(self):
-        """Check that a valid tag but incorrect namespace raises the expected exception."""
+    def test_incorrect_namespace(self) -> None:
+        """Fail to generate text because the tag's namespace is missing."""
         self._test(  # pylint: disable=no-value-for-parameter
-            [u":meth:`path.that.does.not.exist`"], error_classes.MissingNamespace
+            [":meth:`path.that.does.not.exist`"], error_classes.MissingNamespace
         )
 
 
@@ -88,44 +93,46 @@ class ContentsStore(unittest.TestCase):
     """
 
     @staticmethod
-    def _get_fake_project_class():
-        return [u":class:`fake_project.basic.MyKlass`"]
+    def _get_fake_project_class() -> list[str]:
+        return [":class:`fake_project.basic.MyKlass`"]
 
     @staticmethod
-    def _get_fake_project_function():
-        return [u":func:`fake_project.basic.set_function_thing`"]
+    def _get_fake_project_function() -> list[str]:
+        return [":func:`fake_project.basic.set_function_thing`"]
 
     @staticmethod
-    def _get_fake_project_module():
-        return [u":mod:`fake_project.basic`"]
+    def _get_fake_project_module() -> list[str]:
+        return [":mod:`fake_project.basic`"]
 
     @staticmethod
-    def _get_fake_project_nested_class():
-        return [u":class:`fake_project.nested_folder.another.MyKlass`"]
+    def _get_fake_project_nested_class() -> list[str]:
+        return [":class:`fake_project.nested_folder.another.MyKlass`"]
 
     @staticmethod
-    def _get_fake_project_nested_function():
-        return [u":func:`fake_project.nested_folder.another.set_function_thing`"]
+    def _get_fake_project_nested_function() -> list[str]:
+        return [":func:`fake_project.nested_folder.another.set_function_thing`"]
 
     @staticmethod
-    def _get_fake_project_nested_private_function():
-        return [u":func:`fake_project.nested_folder.another._set_private_function_thing`"]
+    def _get_fake_project_nested_private_function() -> list[str]:
+        return [
+            ":func:`fake_project.nested_folder.another._set_private_function_thing`"
+        ]
 
     @staticmethod
-    def _get_fake_project_nested_method():
-        return [u":meth:`fake_project.nested_folder.another.MyKlass.get_method`"]
+    def _get_fake_project_nested_method() -> list[str]:
+        return [":meth:`fake_project.nested_folder.another.MyKlass.get_method`"]
 
     @staticmethod
-    def _get_fake_project_nested_module():
-        return [u":mod:`fake_project.nested_folder.another`"]
+    def _get_fake_project_nested_module() -> list[str]:
+        return [":mod:`fake_project.nested_folder.another`"]
 
     @staticmethod
-    def _get_fake_project_private_function():
-        return [u":func:`fake_project.basic._set_private_function_thing`"]
+    def _get_fake_project_private_function() -> list[str]:
+        return [":func:`fake_project.basic._set_private_function_thing`"]
 
     @staticmethod
-    def _get_fake_project_method():
-        return [u":meth:`fake_project.basic.MyKlass.get_method`"]
+    def _get_fake_project_method() -> list[str]:
+        return [":meth:`fake_project.basic.MyKlass.get_method`"]
 
 
 class Linking(ContentsStore):
@@ -136,9 +143,15 @@ class Linking(ContentsStore):
     @mock.patch("code_include.source_code._get_source_code_from_object")
     @mock.patch("code_include.source_code._get_app_inventory")
     def _get_nodes(
-        data, content, _inventory, _get_from_object, _get_source_module_data
-    ):
-        cache = common.load_cache(os.path.join(_CURRENT_DIRECTORY, "fake_project", "objects.inv"))
+        data: str,
+        content: list[str],
+        _inventory: mock.MagicMock,
+        _get_from_object: mock.MagicMock,
+        _get_source_module_data: mock.MagicMock,
+    ) -> list[nodes_.literal_block]:
+        cache = common.load_cache(
+            os.path.join(_CURRENT_DIRECTORY, "fake_project", "objects.inv")
+        )
 
         _inventory.return_value = cache
         _get_source_module_data.return_value = data
@@ -153,18 +166,18 @@ class Linking(ContentsStore):
     @mock.patch("code_include.extension.Directive._needs_unindent")
     def test_link_to_documentation(
         self,
-        _needs_unindent,
-        _is_link_requested,
-        _get_source_code_from_object,
-    ):
+        _needs_unindent: mock.MagicMock,
+        _is_link_requested: mock.MagicMock,
+        _get_source_code_from_object: mock.MagicMock,
+    ) -> None:
         """Link to the original page where Python source-code was found.
 
         Args:
-            _needs_unindent (:class:`mock.mock.MagicMock`):
+            _needs_unindent:
                 The patched function that controls indentation of code-include.
-            _is_link_requested (:class:`mock.mock.MagicMock`):
+            _is_link_requested:
                 This adds a hyperlink to the original Python documentation.
-            _get_source_code_from_object (:class:`mock.mock.MagicMock`):
+            _get_source_code_from_object:
                 Disable reading from source-code. This forces
                 sphinx-code-include to read from a Sphinx inventory
                 file.
@@ -188,28 +201,34 @@ class Linking(ContentsStore):
         nodes = self._get_nodes(data, content)  # pylint: disable=no-value-for-parameter
 
         self.assertEqual(2, len(nodes))
-        self.assertTrue(any(node for node in nodes if isinstance(
-            node,
-            extension._DocumentationHyperlink,  # pylint: disable=protected-access
-        )))
+        self.assertTrue(
+            any(
+                node
+                for node in nodes
+                if isinstance(
+                    node,
+                    extension._DocumentationHyperlink,  # pylint: disable=protected-access
+                )
+            )
+        )
 
     @mock.patch("code_include.source_code._get_source_code_from_object")
     @mock.patch("code_include.extension.Directive._is_source_requested")
     @mock.patch("code_include.extension.Directive._needs_unindent")
     def test_link_to_source(
         self,
-        _needs_unindent,
-        _is_source_requested,
-        _get_source_code_from_object,
-    ):
+        _needs_unindent: mock.MagicMock,
+        _is_source_requested: mock.MagicMock,
+        _get_source_code_from_object: mock.MagicMock,
+    ) -> None:
         """Link to the original page where Python source-code was found.
 
         Args:
-            _needs_unindent (:class:`mock.mock.MagicMock`):
+            _needs_unindent:
                 The patched function that controls indentation of code-include.
-            _is_source_requested (:class:`mock.mock.MagicMock`):
+            _is_source_requested:
                 This adds a hyperlink to the original Python source-code.
-            _get_source_code_from_object (:class:`mock.mock.MagicMock`):
+            _get_source_code_from_object:
                 Disable reading from source-code. This forces
                 sphinx-code-include to read from a Sphinx inventory
                 file.
@@ -233,10 +252,16 @@ class Linking(ContentsStore):
         nodes = self._get_nodes(data, content)  # pylint: disable=no-value-for-parameter
 
         self.assertEqual(2, len(nodes))
-        self.assertTrue(any(node for node in nodes if isinstance(
-            node,
-            extension._SourceCodeHyperlink,  # pylint: disable=protected-access
-        )))
+        self.assertTrue(
+            any(
+                node
+                for node in nodes
+                if isinstance(
+                    node,
+                    extension._SourceCodeHyperlink,  # pylint: disable=protected-access
+                )
+            )
+        )
 
 
 class _Common(ContentsStore):
@@ -246,30 +271,38 @@ class _Common(ContentsStore):
     @mock.patch("code_include.source_code._get_source_code_from_object")
     @mock.patch("code_include.source_code._get_app_inventory")
     def _test(
-        self, data, content, expected, _inventory, _get_from_object, _get_source_module_data
-    ):
+        self,
+        data: tuple[str, str],
+        content: list[str],
+        expected: str,
+        _inventory: mock.MagicMock,
+        _get_from_object: mock.MagicMock,
+        _get_source_module_data: mock.MagicMock,
+    ) -> None:
         """A generic function that tests a code-include directive for some text.
 
         Args:
-            data (tuple[str, str]):
+            data:
                 The absolute path to an HTML file and the "#foo" tag that
                 would normally be used as a permalink to some header in the
                 HTML file.
-            content (list[str]):
+            content:
                 The lines that the user provides in a standard code-include block.
-            expected (str):
+            expected:
                 The converted source-code text that will be tested for.
-            _inventory (:class:`mock.mock.MagicMock`):
+            _inventory:
                 The function that's normally used to query a Sphinx
                 project's inventory to find every HTML file-path and
                 tag-able header.
-            _get_source_module_data (:class:`mock.mock.MagicMock`):
+            _get_source_module_data:
                 A function that is mocked so that we can skip some of
                 the less important tag-parsing functions and get to the
                 point of this function - testing generated source-code.
 
         """
-        cache = common.load_cache(os.path.join(_CURRENT_DIRECTORY, "fake_project", "objects.inv"))
+        cache = common.load_cache(
+            os.path.join(_CURRENT_DIRECTORY, "fake_project", "objects.inv")
+        )
 
         _inventory.return_value = cache
         _get_source_module_data.return_value = data
@@ -277,16 +310,16 @@ class _Common(ContentsStore):
 
         directive = common.make_mock_directive(content)
         nodes = directive.run()
+        found = str(nodes[0].astext())
 
-        self.assertNotEqual([], nodes)
         self.assertEqual(1, len(nodes))
-        self.assertEqual(expected, nodes[0].astext())
+        self.assertEqual(expected.split("\n"), found.split("\n"))
 
 
 class RenderText(_Common):
     """A class that checks to make sure projects get and return the right code."""
 
-    def test_get_from_html(self):
+    def test_get_from_html(self) -> None:
         """Check that a basic HTML file can be read."""
         data = (
             os.path.join(
@@ -309,7 +342,7 @@ class RenderText(_Common):
 
         self._test(data, content, expected)  # pylint: disable=no-value-for-parameter
 
-    def test_class(self):
+    def test_class(self) -> None:
         """Check that a class is read properly."""
         data = (
             os.path.join(
@@ -360,7 +393,7 @@ class RenderText(_Common):
 
         self._test(data, content, expected)  # pylint: disable=no-value-for-parameter
 
-    def test_function(self):
+    def test_function(self) -> None:
         """Check that a module's function is read properly."""
         data = (
             os.path.join(
@@ -375,22 +408,17 @@ class RenderText(_Common):
         content = self._get_fake_project_function()
 
         expected = textwrap.dedent(
-            '''\
+            """\
             def set_function_thing(value, another):
-                """Do something here."""
-                # Do something with these values
-                # and more comment text, here.
-                #
                 if value:
                     return 2
 
-                # Another comment
-                return 1'''
+                return 1"""
         )
 
         self._test(data, content, expected)  # pylint: disable=no-value-for-parameter
 
-    def test_private_function(self):
+    def test_private_function(self) -> None:
         """Check that a module's function is read properly."""
         data = (
             os.path.join(
@@ -420,7 +448,7 @@ class RenderText(_Common):
 
         self._test(data, content, expected)  # pylint: disable=no-value-for-parameter
 
-    def test_module(self):
+    def test_module(self) -> None:
         """Check that a module is read properly."""
         data = (
             os.path.join(
@@ -548,14 +576,9 @@ class RenderText(_Common):
 
 
             def set_function_thing(value, another):
-                """Do something here."""
-                # Do something with these values
-                # and more comment text, here.
-                #
                 if value:
                     return 2
 
-                # Another comment
                 return 1'''
         )
 
@@ -565,7 +588,7 @@ class RenderText(_Common):
 class RenderTextNested(_Common):
     """A class that checks to make sure projects get and return the right code."""
 
-    def test_get_from_html(self):
+    def test_get_from_html(self) -> None:
         """Check that a basic HTML file can be read."""
         data = (
             os.path.join(
@@ -589,7 +612,7 @@ class RenderTextNested(_Common):
 
         self._test(data, content, expected)  # pylint: disable=no-value-for-parameter
 
-    def test_class(self):
+    def test_class(self) -> None:
         """Check that a class is read properly."""
         data = (
             os.path.join(
@@ -642,7 +665,7 @@ class RenderTextNested(_Common):
 
         self._test(data, content, expected)  # pylint: disable=no-value-for-parameter
 
-    def test_function(self):
+    def test_function(self) -> None:
         """Check that a module's function is read properly."""
         data = (
             os.path.join(
@@ -658,22 +681,17 @@ class RenderTextNested(_Common):
         content = self._get_fake_project_nested_function()
 
         expected = textwrap.dedent(
-            '''\
+            """\
             def set_function_thing(value, another):
-                """Do something here."""
-                # Do something with these values
-                # and more comment text, here.
-                #
                 if value:
                     return 2
 
-                # Another comment
-                return 1'''
+                return 1"""
         )
 
         self._test(data, content, expected)  # pylint: disable=no-value-for-parameter
 
-    def test_private_function(self):
+    def test_private_function(self) -> None:
         """Check that a module's function is read properly."""
         data = (
             os.path.join(
@@ -704,7 +722,7 @@ class RenderTextNested(_Common):
 
         self._test(data, content, expected)  # pylint: disable=no-value-for-parameter
 
-    def test_module(self):
+    def test_module(self) -> None:
         """Check that a module is read properly."""
         data = (
             os.path.join(
@@ -834,14 +852,9 @@ class RenderTextNested(_Common):
 
 
             def set_function_thing(value, another):
-                """Do something here."""
-                # Do something with these values
-                # and more comment text, here.
-                #
                 if value:
                     return 2
 
-                # Another comment
                 return 1'''
         )
 
@@ -849,15 +862,53 @@ class RenderTextNested(_Common):
 
 
 class Options(_Common):
-    """A generic class that tests options that users can add to a code-include directive."""
+    """Make sure code-include directive options work for :obj: tags."""
+
+    @mock.patch("code_include.source_code._get_app_inventory")
+    def test_fallback_text_missing(
+        self,
+        _get_app_inventory: mock.MagicMock,
+    ) -> None:
+        """Raise an exception if the namespace is missing and no fallback is given."""
+        _get_app_inventory.return_value = {"fake": {"thing": ["stuff"]}}
+
+        content = [":meth:`path.that.does.not.exist`"]
+        directive = common.make_mock_directive(content, options={"fallback-text": ""})
+
+        self.assertFalse(directive.run())
+
+    @mock.patch("code_include.source_code._get_app_inventory")
+    def test_fallback_text_simple(
+        self,
+        _get_app_inventory: mock.MagicMock,
+    ) -> None:
+        """Show fallback text if the namespace is missing and fallback text is given."""
+        fallback = "Some fallback text"
+        _get_app_inventory.return_value = {}
+
+        content = [":meth:`path.that.does.not.exist`"]
+        directive = common.make_mock_directive(
+            content,
+            options={"fallback-text": fallback},
+        )
+
+        results = directive.run()
+
+        self.assertEqual(1, len(results))
+        literal_block = results[0]
+        self.assertEqual(fallback, literal_block.astext())
 
     @mock.patch("code_include.source_code._get_source_code_from_object")
     @mock.patch("code_include.extension.Directive._needs_unindent")
-    def test_no_unindent(self, _needs_unindent, _get_source_code_from_object):
+    def test_no_unindent(
+        self,
+        _needs_unindent: mock.MagicMock,
+        _get_source_code_from_object: mock.MagicMock,
+    ) -> None:
         """Check that code-include doesn't remove leading whitespace, when selected.
 
         Args:
-            _needs_unindent (:class:`mock.mock.MagicMock`):
+            _needs_unindent:
                 The patched function that controls indentation of code-include.
 
         """
@@ -885,9 +936,14 @@ class Options(_Common):
 
     @mock.patch("code_include.source_code._get_source_code_from_object")
     @mock.patch("code_include.source_code._get_page_preprocessor")
-    def test_preprocessor(self, _get_page_preprocessor, _get_source_code_from_object):
+    def test_preprocessor(
+        self,
+        _get_page_preprocessor: mock.MagicMock,
+        _get_source_code_from_object: mock.MagicMock,
+    ) -> None:
         """Check that the optional user-configuration function works correctly."""
-        def _remove_comments_and_docstrings(node):
+
+        def _remove_comments_and_docstrings(node: bs4.BeautifulSoup) -> None:
             for tag in node.find_all("span", {"class": ["c1", "sd"]}):
                 tag.decompose()
 
@@ -906,16 +962,11 @@ class Options(_Common):
         )
         content = self._get_fake_project_function()
 
-        expected = '''\
+        expected = """\
 def set_function_thing(value, another):
-    
-    
-    
-    
     if value:
         return 2
 
-    
-    return 1'''
+    return 1"""
 
         self._test(data, content, expected)  # pylint: disable=no-value-for-parameter
